@@ -173,7 +173,9 @@ editor_application_actions_new_window_cb (GSimpleAction *action,
   current = editor_application_get_current_window (self);
   session = editor_application_get_session (self);
 
-  if (current != NULL && !editor_window_get_visible_page (current))
+  if (current == NULL)
+    g_application_activate (G_APPLICATION (self));
+  else if (current != NULL && !editor_window_get_visible_page (current))
     editor_session_add_draft (session, current);
   else
     editor_session_create_window (session);
