@@ -617,6 +617,17 @@ editor_document_set_encoding (EditorDocument *self,
   _editor_document_set_encoding (self, enc);
 }
 
+static void
+editor_document_notify_location_cb (EditorDocument *self,
+                                    GParamSpec     *pspec,
+                                    GtkSourceFile  *file)
+{
+  g_assert (EDITOR_IS_DOCUMENT (self));
+  g_assert (GTK_SOURCE_IS_FILE (file));
+
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_FILE]);
+}
+
 static void
 editor_document_constructed (GObject *object)
 {
@@ -902,6 +913,12 @@ editor_document_init (EditorDocument *self)
   self->file = gtk_source_file_new ();
   self->draft_id = g_uuid_string_random ();
 
+  g_signal_connect_object (self->file,
+                           "notify::location",
+                           G_CALLBACK (editor_document_notify_location_cb),
+                           self,
+                           G_CONNECT_SWAPPED);
+
   g_signal_connect (self,
                     "cursor-moved",
                     G_CALLBACK (editor_document_cursor_moved),
@@ -2203,6 +2220,7 @@ editor_document_dup_title (EditorDocument *self)
 
       do
         {
+          GUnicodeScript script;
           gunichar ch;
 
           if (count >= TITLE_MAX_LEN)
@@ -2220,7 +2238,10 @@ editor_document_dup_title (EditorDocument *self)
                 break;
             }
 
-          if (g_unichar_isspace (ch) || !g_unichar_isalnum (ch))
+          script = g_unichar_get_script (ch);
+
+          if (g_unichar_isspace (ch) ||
+              (((script == G_UNICODE_SCRIPT_COMMON) || (script == G_UNICODE_SCRIPT_LATIN)) && !g_unichar_isalnum (ch)))
             {
               if (count > TITLE_LAST_WORD_POS)
                 break;
