@@ -28,9 +28,9 @@ G_BEGIN_DECLS
 
 #define EDITOR_TYPE_LANGUAGE_DIALOG (editor_language_dialog_get_type())
 
-G_DECLARE_FINAL_TYPE (EditorLanguageDialog, editor_language_dialog, EDITOR, LANGUAGE_DIALOG, AdwWindow)
+G_DECLARE_FINAL_TYPE (EditorLanguageDialog, editor_language_dialog, EDITOR, LANGUAGE_DIALOG, AdwDialog)
 
-EditorLanguageDialog *editor_language_dialog_new          (EditorApplication    *application);
+EditorLanguageDialog *editor_language_dialog_new          (void);
 GtkSourceLanguage    *editor_language_dialog_get_language (EditorLanguageDialog *self);
 void                  editor_language_dialog_set_language (EditorLanguageDialog *self,
                                                            GtkSourceLanguage    *language);
