@@ -255,7 +255,6 @@ _editor_save_changes_dialog_new (GtkWindow *parent,
 
   dialog = adw_alert_dialog_new (_("Save Changes?"),
                                  _("Open documents contain unsaved changes. Changes which are not saved will be permanently lost."));
-  gtk_widget_add_css_class (GTK_WIDGET (dialog), "save-changes");
 
   adw_alert_dialog_add_responses (ADW_ALERT_DIALOG (dialog),
                                   "cancel", _("_Cancel"),
@@ -317,7 +316,7 @@ _editor_save_changes_dialog_new (GtkWindow *parent,
                             "valign", GTK_ALIGN_CENTER,
                             NULL);
 
-      gtk_accessible_update_property (GTK_ACCESSIBLE (check), GTK_ACCESSIBLE_PROPERTY_LABEL, _("Save changes for this document"), -1);
+      adw_preferences_row_set_use_markup (ADW_PREFERENCES_ROW (row), FALSE);
       adw_preferences_row_set_title (ADW_PREFERENCES_ROW (row), title_str);
       adw_action_row_set_subtitle (ADW_ACTION_ROW (row), subtitle_str);
       adw_action_row_add_prefix (ADW_ACTION_ROW (row), check);
