@@ -281,6 +281,7 @@ _editor_search_bar_move_next (EditorSearchBar *self,
   buffer = gtk_source_search_context_get_buffer (self->context);
   gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (buffer), &begin, &end);
   gtk_text_iter_order (&begin, &end);
+
   reset_cancellable (self);
 
   gtk_source_search_context_forward_async (self->context,
@@ -309,6 +310,7 @@ _editor_search_bar_move_previous (EditorSearchBar *self,
   buffer = gtk_source_search_context_get_buffer (self->context);
   gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (buffer), &begin, &end);
   gtk_text_iter_order (&begin, &end);
+
   reset_cancellable (self);
 
   gtk_source_search_context_backward_async (self->context,
@@ -319,6 +321,42 @@ _editor_search_bar_move_previous (EditorSearchBar *self,
                                             weak_hold_new (self));
 }
 
+static void
+search_activate (GtkWidget  *widget,
+                 const char *action_name,
+                 GVariant   *param)
+{
+  EditorSearchBar *self = EDITOR_SEARCH_BAR (widget);
+
+  if (self->context == NULL)
+    return;
+
+  if (editor_search_entry_get_occurrence_position (self->search_entry) > 0)
+    gtk_widget_activate_action (GTK_WIDGET (self), "search.hide", NULL);
+  else
+    _editor_search_bar_move_next (self, TRUE);
+}
+
+static void
+search_cancel (GtkWidget  *widget,
+               const char *action_name,
+               GVariant   *param)
+{
+  EditorSearchBar *self = EDITOR_SEARCH_BAR (widget);
+  EditorDocument *document;
+  EditorPage *page;
+
+  if (!(page = EDITOR_PAGE (gtk_widget_get_ancestor (widget, EDITOR_TYPE_PAGE))))
+    return;
+
+  document = editor_page_get_document (page);
+  _editor_document_restore_insert_mark (document);
+
+  gtk_widget_activate_action (GTK_WIDGET (self), "search.hide", NULL);
+
+  _editor_page_scroll_to_insert (page);
+}
+
 static gboolean
 text_to_search_text (GBinding     *binding,
                      const GValue *from_value,
@@ -449,21 +487,37 @@ editor_search_bar_focus (GtkWidget        *widget,
                          GtkDirectionType  dir)
 {
   EditorSearchBar *self = EDITOR_SEARCH_BAR (widget);
+  GtkWidget *focus;
+  GtkWidget *popover;
 
-  return _editor_focus_chain_focus_child (widget, &self->focus_chain, dir);
+  /* If the focus is in the menu popover then use the default
+   * movements so we don't ness that up.
+   */
+  focus = gtk_root_get_focus (gtk_widget_get_root (widget));
+  if (focus != NULL &&
+      gtk_widget_is_ancestor (focus, widget) &&
+      (popover = gtk_widget_get_ancestor (focus, GTK_TYPE_POPOVER)))
+    return GTK_WIDGET_CLASS (editor_search_bar_parent_class)->focus (widget, dir);
+
+  if (_editor_focus_chain_focus_child (widget, &self->focus_chain, dir))
+    return TRUE;
+
+  return GTK_WIDGET_CLASS (editor_search_bar_parent_class)->focus (widget, dir);
 }
 
 static void
 editor_search_bar_dispose (GObject *object)
 {
   EditorSearchBar *self = (EditorSearchBar *)object;
+  GtkWidget *child;
 
   g_cancellable_cancel (self->cancellable);
   g_clear_object (&self->context);
 
   gtk_widget_dispose_template (GTK_WIDGET (self), EDITOR_TYPE_SEARCH_BAR);
 
-  g_clear_pointer ((GtkWidget **)&self->grid, gtk_widget_unparent);
+  while ((child = gtk_widget_get_first_child (GTK_WIDGET (self))))
+    gtk_widget_unparent (child);
 
   if (self->focus_chain.length > 0)
     g_queue_clear (&self->focus_chain);
@@ -554,20 +608,16 @@ editor_search_bar_class_init (EditorSearchBarClass *klass)
   gtk_widget_class_set_layout_manager_type (widget_class, GTK_TYPE_BIN_LAYOUT);
   gtk_widget_class_set_css_name (widget_class, "searchbar");
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/TextEditor/ui/editor-search-bar.ui");
-  gtk_widget_class_bind_template_child (widget_class, EditorSearchBar, case_button);
   gtk_widget_class_bind_template_child (widget_class, EditorSearchBar, close_button);
   gtk_widget_class_bind_template_child (widget_class, EditorSearchBar, grid);
   gtk_widget_class_bind_template_child (widget_class, EditorSearchBar, move_previous);
   gtk_widget_class_bind_template_child (widget_class, EditorSearchBar, move_next);
-  gtk_widget_class_bind_template_child (widget_class, EditorSearchBar, options_box);
   gtk_widget_class_bind_template_child (widget_class, EditorSearchBar, options_button);
-  gtk_widget_class_bind_template_child (widget_class, EditorSearchBar, regex_button);
   gtk_widget_class_bind_template_child (widget_class, EditorSearchBar, replace_all_button);
   gtk_widget_class_bind_template_child (widget_class, EditorSearchBar, replace_button);
   gtk_widget_class_bind_template_child (widget_class, EditorSearchBar, replace_entry);
   gtk_widget_class_bind_template_child (widget_class, EditorSearchBar, replace_mode_button);
   gtk_widget_class_bind_template_child (widget_class, EditorSearchBar, search_entry);
-  gtk_widget_class_bind_template_child (widget_class, EditorSearchBar, word_button);
   gtk_widget_class_bind_template_callback (widget_class, on_search_key_pressed_cb);
 
   signals [MOVE_NEXT_SEARCH] =
@@ -588,7 +638,10 @@ editor_search_bar_class_init (EditorSearchBarClass *klass)
                                 NULL,
                                 G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
 
-  gtk_widget_class_add_binding_action (widget_class, GDK_KEY_Escape, 0, "search.hide", NULL);
+  gtk_widget_class_add_binding_action (widget_class, GDK_KEY_Escape, 0, "search.cancel", NULL);
+
+  gtk_widget_class_install_action (widget_class, "search.activate", NULL, search_activate);
+  gtk_widget_class_install_action (widget_class, "search.cancel", NULL, search_cancel);
 
   properties [PROP_MODE] =
     g_param_spec_enum ("mode",
@@ -627,6 +680,11 @@ editor_search_bar_class_init (EditorSearchBarClass *klass)
 static void
 editor_search_bar_init (EditorSearchBar *self)
 {
+  g_autoptr(GSimpleActionGroup) group = NULL;
+  g_autoptr(GPropertyAction) regex_action = NULL;
+  g_autoptr(GPropertyAction) case_action = NULL;
+  g_autoptr(GPropertyAction) word_action = NULL;
+
   self->cancellable = g_cancellable_new ();
 
   gtk_widget_init_template (GTK_WIDGET (self));
@@ -638,12 +696,9 @@ editor_search_bar_init (EditorSearchBar *self)
   g_queue_push_tail (&self->focus_chain, GTK_WIDGET (self->move_next));
   g_queue_push_tail (&self->focus_chain, GTK_WIDGET (self->replace_mode_button));
   g_queue_push_tail (&self->focus_chain, GTK_WIDGET (self->options_button));
+  g_queue_push_tail (&self->focus_chain, GTK_WIDGET (self->close_button));
   g_queue_push_tail (&self->focus_chain, GTK_WIDGET (self->replace_button));
   g_queue_push_tail (&self->focus_chain, GTK_WIDGET (self->replace_all_button));
-  g_queue_push_tail (&self->focus_chain, GTK_WIDGET (self->regex_button));
-  g_queue_push_tail (&self->focus_chain, GTK_WIDGET (self->case_button));
-  g_queue_push_tail (&self->focus_chain, GTK_WIDGET (self->word_button));
-  g_queue_push_tail (&self->focus_chain, GTK_WIDGET (self->close_button));
 
   g_signal_connect_object (self->replace_entry,
                            "notify::text",
@@ -669,19 +724,17 @@ editor_search_bar_init (EditorSearchBar *self)
                                self, "mode",
                                G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL,
                                boolean_to_mode, mode_to_boolean, NULL, NULL);
-  g_object_bind_property (self->settings, "at-word-boundaries",
-                          self->word_button, "active",
-                          G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
-  g_object_bind_property (self->settings, "regex-enabled",
-                          self->regex_button, "active",
-                          G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
-  g_object_bind_property (self->settings, "case-sensitive",
-                          self->case_button, "active",
-                          G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
 
-  g_object_bind_property (self->options_button, "active",
-                          self->options_box, "visible",
-                          G_BINDING_SYNC_CREATE);
+  regex_action = g_property_action_new ("regex", G_OBJECT (self->settings), "regex-enabled");
+  case_action = g_property_action_new ("case-sensitive", G_OBJECT (self->settings), "case-sensitive");
+  word_action = g_property_action_new ("match-whole-word", G_OBJECT (self->settings), "at-word-boundaries");
+
+  group = g_simple_action_group_new ();
+  g_action_map_add_action (G_ACTION_MAP (group), G_ACTION (regex_action));
+  g_action_map_add_action (G_ACTION_MAP (group), G_ACTION (case_action));
+  g_action_map_add_action (G_ACTION_MAP (group), G_ACTION (word_action));
+
+  gtk_widget_insert_action_group (GTK_WIDGET (self), "search-options", G_ACTION_GROUP (group));
 }
 
 void
@@ -722,7 +775,7 @@ scroll_to_first_match (EditorSearchBar        *self,
   if (gtk_source_search_context_forward (context, &iter, &match_begin, &match_end, &wrapped))
     {
       gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW (EDITOR_PAGE (page)->view),
-                                    &match_begin, 0.25, TRUE, 1.0, 0.5);
+                                    &match_begin, 0.1, FALSE, 0, 0);
       self->jump_back_on_hide = TRUE;
     }
 
@@ -745,6 +798,12 @@ editor_search_bar_notify_occurrences_count_cb (EditorSearchBar        *self,
   if (self->scroll_to_first_match && occurrence_count > 0)
     scroll_to_first_match (self, context);
 
+  if (occurrence_count == 0 &&
+      g_strcmp0 (gtk_editable_get_text (GTK_EDITABLE (self->search_entry)), "") != 0)
+    gtk_widget_add_css_class (GTK_WIDGET (self->search_entry), "error");
+  else
+    gtk_widget_remove_css_class (GTK_WIDGET (self->search_entry), "error");
+
   update_properties (self);
 }
 
@@ -767,6 +826,8 @@ _editor_search_bar_attach (EditorSearchBar *self,
 
   g_return_if_fail (EDITOR_IS_SEARCH_BAR (self));
 
+  _editor_document_save_insert_mark (document);
+
   gtk_text_buffer_get_iter_at_mark (buffer, &insert, gtk_text_buffer_get_insert (buffer));
   self->offset_when_shown = gtk_text_iter_get_offset (&insert);
 
