@@ -28,6 +28,8 @@ G_BEGIN_DECLS
 
 char                    *_editor_font_description_to_css        (const PangoFontDescription *font_desc);
 void                     _editor_widget_hide_with_fade          (GtkWidget                  *widget);
+void                     _editor_widget_hide_with_fade_delay    (GtkWidget                  *widget,
+                                                                 guint                       delay);
 gboolean                 _editor_gchararray_to_boolean          (GBinding                   *binding,
                                                                  const GValue               *from_value,
                                                                  GValue                     *to_value,
@@ -49,12 +51,6 @@ gboolean                 _editor_gchararray_to_style_scheme     (GBinding
                                                                  GValue                     *to_value,
                                                                  gpointer                    user_data);
 char                    *_editor_date_time_format               (GDateTime                  *self);
-void                     _editor_file_chooser_add_encodings     (GtkFileChooser             *chooser,
-                                                                 const GtkSourceEncoding    *selected);
-void                     _editor_file_chooser_add_line_endings  (GtkFileChooser             *chooser,
-                                                                 GtkSourceNewlineType        selected);
-const GtkSourceEncoding *_editor_file_chooser_get_encoding      (GtkFileChooser             *chooser);
-GtkSourceNewlineType     _editor_file_chooser_get_line_ending   (GtkFileChooser             *chooser);
 void                     _editor_revealer_auto_hide             (GtkRevealer                *revealer);
 
 G_END_DECLS
