@@ -231,7 +231,8 @@ hide_callback (gpointer data)
 }
 
 void
-_editor_widget_hide_with_fade (GtkWidget *widget)
+_editor_widget_hide_with_fade_delay (GtkWidget *widget,
+                                     guint      delay)
 {
   GdkFrameClock *frame_clock;
   EditorAnimation *anim;
@@ -246,8 +247,8 @@ _editor_widget_hide_with_fade (GtkWidget *widget)
 
       frame_clock = gtk_widget_get_frame_clock (widget);
       anim = editor_object_animate_full (widget,
-                                         EDITOR_ANIMATION_LINEAR,
-                                         1000,
+                                         EDITOR_ANIMATION_EASE_OUT_CUBIC,
+                                         delay,
                                          frame_clock,
                                          hide_callback,
                                          g_object_ref (widget),
@@ -260,6 +261,12 @@ _editor_widget_hide_with_fade (GtkWidget *widget)
     }
 }
 
+void
+_editor_widget_hide_with_fade (GtkWidget *widget)
+{
+  _editor_widget_hide_with_fade_delay (widget, 1000);
+}
+
 gboolean
 _editor_gchararray_to_boolean (GBinding     *binding,
                                const GValue *from_value,
@@ -362,144 +369,6 @@ _editor_date_time_format (GDateTime *self)
   return g_strdup_printf (ngettext ("About %u year ago", "About %u years ago", years), years);
 }
 
-static const struct {
-  GtkSourceNewlineType type;
-  const char *id;
-  const char *label;
-} line_endings[] = {
-  { GTK_SOURCE_NEWLINE_TYPE_LF, "unix", N_("Unix/Linux (LF)") },
-  { GTK_SOURCE_NEWLINE_TYPE_CR, "mac", N_("Mac OS Classic (CR)") },
-  { GTK_SOURCE_NEWLINE_TYPE_CR_LF, "windows", N_("Windows (CR+LF)") },
-};
-
-static int
-sort_by_name (gconstpointer a,
-              gconstpointer b)
-{
-  return g_strcmp0 (gtk_source_encoding_get_name (a),
-                    gtk_source_encoding_get_name (b));
-}
-
-void
-_editor_file_chooser_add_encodings (GtkFileChooser *chooser)
-{
-  GPtrArray *choices;
-  GPtrArray *labels;
-  GSList *all;
-
-  g_assert (GTK_IS_FILE_CHOOSER (chooser));
-
-  all = g_slist_sort (gtk_source_encoding_get_all (), sort_by_name);
-  choices = g_ptr_array_new ();
-  labels = g_ptr_array_new_with_free_func (g_free);
-
-#define ADD_ENCODING(id, name)             \
-  G_STMT_START {                           \
-    g_ptr_array_add(choices, (char *)id);  \
-    g_ptr_array_add(labels, (char *)name); \
-  } G_STMT_END
-
-  ADD_ENCODING ("auto", g_strdup (_("Automatically Detected")));
-
-  for (const GSList *l = all; l; l = l->next)
-    {
-      GtkSourceEncoding *encoding = l->data;
-      char *title = g_strdup_printf ("%s (%s)",
-                                     gtk_source_encoding_get_name (encoding),
-                                     gtk_source_encoding_get_charset (encoding));
-      ADD_ENCODING (gtk_source_encoding_get_charset (encoding), title);
-    }
-
-  ADD_ENCODING (NULL, NULL);
-#undef ADD_ENCODING
-
-  gtk_file_chooser_add_choice (chooser,
-                               "encoding",
-                               _("Character Encoding:"),
-                               (const char **)(gpointer)choices->pdata,
-                               (const char **)(gpointer)labels->pdata);
-  gtk_file_chooser_set_choice (chooser, "encoding", "auto");
-
-  g_slist_free (all);
-  g_clear_pointer (&choices, g_ptr_array_unref);
-  g_clear_pointer (&labels, g_ptr_array_unref);
-}
-
-void
-_editor_file_chooser_add_line_endings (GtkFileChooser       *chooser,
-                                       GtkSourceNewlineType  selected)
-{
-  static GArray *choices;
-  static GArray *labels;
-
-  g_return_if_fail (GTK_IS_FILE_CHOOSER (chooser));
-
-  if (choices == NULL)
-    {
-      choices = g_array_new (TRUE, FALSE, sizeof (char *));
-      labels = g_array_new (TRUE, FALSE, sizeof (char *));
-
-      for (guint i = 0; i < G_N_ELEMENTS (line_endings); i++)
-        {
-          const char *msg = g_dgettext (GETTEXT_PACKAGE, line_endings[i].label);
-
-          g_array_append_val (choices, line_endings[i].id);
-          g_array_append_val (labels, msg);
-        }
-    }
-
-  gtk_file_chooser_add_choice (chooser,
-                               CHOICE_LINE_ENDING,
-                               _("Line Ending:"),
-                               (const char **)(gpointer)choices->data,
-                               (const char **)(gpointer)labels->data);
-  gtk_file_chooser_set_choice (chooser, CHOICE_LINE_ENDING, "unix");
-
-  for (guint i = 0; i < G_N_ELEMENTS (line_endings); i++)
-    {
-      if (line_endings[i].type == selected)
-        {
-          gtk_file_chooser_set_choice (chooser, CHOICE_LINE_ENDING, line_endings[i].id);
-          break;
-        }
-    }
-}
-
-const GtkSourceEncoding *
-_editor_file_chooser_get_encoding (GtkFileChooser *chooser)
-{
-  const char *encoding;
-
-  g_return_val_if_fail (GTK_IS_FILE_CHOOSER (chooser), NULL);
-
-  if ((encoding = gtk_file_chooser_get_choice (chooser, "encoding")))
-    {
-      if (strcmp (encoding, "auto") != 0)
-        return gtk_source_encoding_get_from_charset (encoding);
-    }
-
-  return NULL;
-}
-
-GtkSourceNewlineType
-_editor_file_chooser_get_line_ending (GtkFileChooser *chooser)
-{
-  const char *ending;
-
-  g_return_val_if_fail (GTK_IS_FILE_CHOOSER (chooser), 0);
-
-  if ((ending = gtk_file_chooser_get_choice (chooser, CHOICE_LINE_ENDING)))
-    {
-      for (guint i = 0; i < G_N_ELEMENTS (line_endings); i++)
-        {
-          if (g_strcmp0 (ending, line_endings[i].id) == 0)
-            return line_endings[i].type;
-        }
-    }
-
-  return GTK_SOURCE_NEWLINE_TYPE_LF;
-}
-
 static gboolean
 revealer_autohide (gpointer data)
 {
