@@ -26,10 +26,10 @@
 
 #include "editor-application.h"
 #include "editor-document.h"
+#include "editor-encoding-dialog.h"
 #include "editor-language-dialog.h"
 #include "editor-page-private.h"
 #include "editor-preferences-dialog-private.h"
-#include "editor-properties-dialog-private.h"
 #include "editor-save-changes-dialog-private.h"
 #include "editor-session-private.h"
 #include "editor-window-private.h"
@@ -197,23 +197,7 @@ editor_window_actions_change_language_cb (GtkWidget  *widget,
 }
 
 static void
-editor_window_actions_discard_changes_cb (GtkWidget  *widget,
-                                          const char *action_name,
-                                          GVariant   *param)
-{
-  EditorWindow *self = (EditorWindow *)widget;
-  EditorPage *page;
-
-  g_assert (EDITOR_IS_WINDOW (self));
-
-  if ((page = editor_window_get_visible_page (self)))
-    _editor_page_discard_changes (page);
-}
-
-static void
-editor_window_actions_confirm_discard_changes_cb (GtkWidget  *widget,
-                                                  const char *action_name,
-                                                  GVariant   *param)
+editor_window_actions_confirm_discard_changes (GtkWidget *widget)
 {
   EditorWindow *self = (EditorWindow *)widget;
   g_autofree gchar *title = NULL;
@@ -247,6 +231,34 @@ editor_window_actions_confirm_discard_changes_cb (GtkWidget  *widget,
   adw_dialog_present (dialog, widget);
 }
 
+static void
+editor_window_actions_discard_changes_cb (GtkWidget  *widget,
+                                          const char *action_name,
+                                          GVariant   *param)
+{
+  EditorWindow *self = (EditorWindow *)widget;
+
+  g_assert (EDITOR_IS_WINDOW (self));
+  g_assert (g_variant_is_of_type (param, G_VARIANT_TYPE_BOOLEAN));
+
+  if (!g_variant_get_boolean (param))
+    editor_window_actions_confirm_discard_changes (widget);
+  else
+    _editor_page_discard_changes (editor_window_get_visible_page (self));
+}
+
+static void
+editor_window_actions_infobar_discard_changes_cb (GtkWidget  *widget,
+                                                  const char *action_name,
+                                                  GVariant   *param)
+{
+  EditorWindow *self = (EditorWindow *)widget;
+
+  g_assert (EDITOR_IS_WINDOW (self));
+
+  editor_window_actions_confirm_discard_changes (widget);
+}
+
 static void
 editor_window_actions_print_cb (GtkWidget  *widget,
                                 const char *action_name,
@@ -296,31 +308,34 @@ editor_window_actions_copy_all_cb (GtkWidget  *widget,
 }
 
 static void
-editor_window_actions_open_response_cb (EditorWindow         *self,
-                                        gint                  response_id,
-                                        GtkFileChooserNative *native)
+editor_window_actions_open_text_file_cb (GObject      *object,
+                                         GAsyncResult *result,
+                                         gpointer      user_data)
 {
+  GtkFileDialog *dialog = (GtkFileDialog *)object;
+  g_autoptr(EditorWindow) self = user_data;
+  g_autoptr(GError) error = NULL;
+  g_autoptr(GListModel) files = NULL;
+  const char *encoding = NULL;
+
+  g_assert (GTK_IS_FILE_DIALOG (dialog));
+  g_assert (G_IS_ASYNC_RESULT (result));
   g_assert (EDITOR_IS_WINDOW (self));
-  g_assert (GTK_IS_FILE_CHOOSER_NATIVE (native));
 
-  if (response_id == GTK_RESPONSE_ACCEPT)
+  if ((files = gtk_file_dialog_open_multiple_text_files_finish (dialog, result, &encoding, &error)))
     {
-      g_autoptr(GListModel) files = gtk_file_chooser_get_files (GTK_FILE_CHOOSER (native));
-      guint i = 0;
-      GFile *file = NULL;
-      const GtkSourceEncoding *encoding = _editor_file_chooser_get_encoding (GTK_FILE_CHOOSER (native));
+      const GtkSourceEncoding *translated = NULL;
+      guint n_items = g_list_model_get_n_items (files);
 
-      g_assert (g_list_model_get_item_type (files) == G_TYPE_FILE);
+      if (encoding != NULL && g_strcmp0 (encoding, "automatic") != 0)
+        translated = gtk_source_encoding_get_from_charset (encoding);
 
-      while ((file = G_FILE (g_list_model_get_object (files, i++))))
+      for (guint i = 0; i < n_items; i++)
         {
-          editor_session_open (EDITOR_SESSION_DEFAULT, self, file, encoding);
-          g_object_unref (file);
+          g_autoptr(GFile) file = g_list_model_get_item (files, i);
+          editor_session_open (EDITOR_SESSION_DEFAULT, self, file, translated);
         }
     }
-
-  gtk_native_dialog_destroy (GTK_NATIVE_DIALOG (native));
-  g_object_unref (native);
 }
 
 static void
@@ -331,7 +346,8 @@ editor_window_actions_open_cb (GtkWidget  *widget,
   EditorWindow *self = (EditorWindow *)widget;
   g_autoptr(GtkFileFilter) all_files = NULL;
   g_autoptr(GtkFileFilter) text_files = NULL;
-  GtkFileChooserNative *native;
+  g_autoptr(GtkFileDialog) dialog = NULL;
+  g_autoptr(GListStore) filters = NULL;
   EditorDocument *document;
   EditorPage *page;
   GFile *dfile;
@@ -340,11 +356,8 @@ editor_window_actions_open_cb (GtkWidget  *widget,
 
   gtk_menu_button_popdown (self->open_menu_button);
 
-  native = gtk_file_chooser_native_new (_("Open File"),
-                                        GTK_WINDOW (self),
-                                        GTK_FILE_CHOOSER_ACTION_OPEN,
-                                        _("Open"),
-                                        _("Cancel"));
+  dialog = gtk_file_dialog_new ();
+  filters = g_list_store_new (GTK_TYPE_FILE_FILTER);
 
   if ((page = editor_window_get_visible_page (self)) &&
       (document = editor_page_get_document (page)) &&
@@ -353,7 +366,7 @@ editor_window_actions_open_cb (GtkWidget  *widget,
       g_autoptr(GFile) dir = g_file_get_parent (dfile);
 
       if (dir != NULL)
-        gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (native), dir, NULL);
+        gtk_file_dialog_set_initial_folder (dialog, dir);
     }
   else
     {
@@ -363,39 +376,35 @@ editor_window_actions_open_cb (GtkWidget  *widget,
       if (uri && uri[0])
         {
           g_autoptr(GFile) dir = g_file_new_for_uri (uri);
-          gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (native), dir, NULL);
+          gtk_file_dialog_set_initial_folder (dialog, dir);
         }
     }
 
   all_files = gtk_file_filter_new ();
   gtk_file_filter_set_name (all_files, _("All Files"));
   gtk_file_filter_add_pattern (all_files, "*");
-  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (native), g_object_ref (all_files));
+  g_list_store_append (filters, all_files);
 
   text_files = gtk_file_filter_new ();
   gtk_file_filter_set_name (text_files, _("Text Files"));
   gtk_file_filter_add_mime_type (text_files, "text/plain");
   gtk_file_filter_add_mime_type (text_files, "application/x-zerosize");
-  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (native), g_object_ref (text_files));
+  g_list_store_append (filters, text_files);
+
+  gtk_file_dialog_set_filters (dialog, G_LIST_MODEL (filters));
 
 #ifdef __APPLE__
   /* Apple content-type detect is pretty bad */
-  gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (native), all_files);
+  gtk_file_dialog_set_default_filter (dialog, all_files);
 #else
-  gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (native), text_files);
+  gtk_file_dialog_set_default_filter (dialog, text_files);
 #endif
 
-  _editor_file_chooser_add_encodings (GTK_FILE_CHOOSER (native), NULL);
-
-  gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (native), TRUE);
-
-  g_signal_connect_object (native,
-                           "response",
-                           G_CALLBACK (editor_window_actions_open_response_cb),
-                           self,
-                           G_CONNECT_SWAPPED);
-
-  gtk_native_dialog_show (GTK_NATIVE_DIALOG (native));
+  gtk_file_dialog_open_multiple_text_files (dialog,
+                                            GTK_WINDOW (self),
+                                            NULL,
+                                            editor_window_actions_open_text_file_cb,
+                                            g_object_ref (self));
 }
 
 static void
@@ -555,6 +564,26 @@ editor_window_actions_close_other_pages_cb (GtkWidget  *widget,
   g_list_free (pages);
 }
 
+static void
+editor_window_actions_change_encoding_cb (GtkWidget  *widget,
+                                          const char *action_name,
+                                          GVariant   *param)
+{
+  EditorWindow *self = (EditorWindow *)widget;
+  EditorDocument *document;
+  EditorPage *page;
+
+  g_assert (EDITOR_IS_WINDOW (self));
+
+  if ((page = editor_window_get_visible_page (self)) &&
+      (document = editor_page_get_document (page)))
+    {
+      AdwDialog *dialog = editor_encoding_dialog_new (document);
+
+      adw_dialog_present (dialog, GTK_WIDGET (self));
+    }
+}
+
 static void
 editor_window_actions_page_zoom_in_cb (GtkWidget  *widget,
                                        const char *action_name,
@@ -591,23 +620,6 @@ editor_window_actions_page_zoom_one_cb (GtkWidget  *widget,
   _editor_page_zoom_one (editor_window_get_visible_page (self));
 }
 
-static void
-editor_window_actions_page_properties_cb (GtkWidget  *widget,
-                                          const char *action_name,
-                                          GVariant   *param)
-{
-  EditorWindow *self = (EditorWindow *)widget;
-  EditorDocument *document;
-  GtkWidget *window;
-
-  g_assert (EDITOR_IS_WINDOW (self));
-
-  document = editor_page_get_document (self->visible_page);
-  window = editor_properties_dialog_new (self, document);
-
-  gtk_window_present (GTK_WINDOW (window));
-}
-
 void
 _editor_window_class_actions_init (EditorWindowClass *klass)
 {
@@ -663,12 +675,12 @@ _editor_window_class_actions_init (EditorWindowClass *klass)
                                    editor_window_actions_change_language_cb);
   gtk_widget_class_install_action (widget_class,
                                    "page.discard-changes",
-                                   NULL,
+                                   "b",
                                    editor_window_actions_discard_changes_cb);
   gtk_widget_class_install_action (widget_class,
-                                   "page.confirm-discard-changes",
+                                   "page.infobar-discard-changes",
                                    NULL,
-                                   editor_window_actions_confirm_discard_changes_cb);
+                                   editor_window_actions_infobar_discard_changes_cb);
   gtk_widget_class_install_action (widget_class,
                                    "page.print",
                                    NULL,
@@ -705,6 +717,10 @@ _editor_window_class_actions_init (EditorWindowClass *klass)
                                    "win.show-preferences",
                                    NULL,
                                    editor_window_actions_show_preferences_cb);
+  gtk_widget_class_install_action (widget_class,
+                                   "page.change-encoding",
+                                   NULL,
+                                   editor_window_actions_change_encoding_cb);
   gtk_widget_class_install_action (widget_class,
                                    "page.zoom-in",
                                    NULL,
@@ -717,10 +733,6 @@ _editor_window_class_actions_init (EditorWindowClass *klass)
                                    "page.zoom-one",
                                    NULL,
                                    editor_window_actions_page_zoom_one_cb);
-  gtk_widget_class_install_action (widget_class,
-                                   "page.properties",
-                                   NULL,
-                                   editor_window_actions_page_properties_cb);
 }
 
 void
@@ -768,6 +780,7 @@ _editor_window_actions_update (EditorWindow *self,
   gboolean can_save = FALSE;
   gboolean modified = FALSE;
   gboolean draft = FALSE;
+  int font_scale = 0;
 
   g_assert (EDITOR_IS_WINDOW (self));
   g_assert (!page || EDITOR_IS_PAGE (page));
@@ -781,6 +794,7 @@ _editor_window_actions_update (EditorWindow *self,
       modified = editor_page_get_is_modified (page);
       draft = editor_page_is_draft (page);
       externally_modified = editor_document_get_externally_modified (document);
+      g_object_get (page->view, "font-scale", &font_scale, NULL);
     }
 
   gtk_widget_action_set_enabled (GTK_WIDGET (self), "win.close-current-page", has_page);
@@ -788,7 +802,7 @@ _editor_window_actions_update (EditorWindow *self,
   gtk_widget_action_set_enabled (GTK_WIDGET (self), "page.change-language", has_page);
   gtk_widget_action_set_enabled (GTK_WIDGET (self), "page.discard-changes", externally_modified || (modified && !draft));
   gtk_widget_action_set_enabled (GTK_WIDGET (self), "page.print", has_page);
-  gtk_widget_action_set_enabled (GTK_WIDGET (self), "page.properties", has_page);
+  gtk_widget_action_set_enabled (GTK_WIDGET (self), "page.change-encoding", has_page);
   gtk_widget_action_set_enabled (GTK_WIDGET (self), "page.save", can_save);
   gtk_widget_action_set_enabled (GTK_WIDGET (self), "page.save-as", has_page);
   gtk_widget_action_set_enabled (GTK_WIDGET (self), "page.copy-all", has_page);
@@ -796,6 +810,11 @@ _editor_window_actions_update (EditorWindow *self,
   gtk_widget_action_set_enabled (GTK_WIDGET (self), "page.begin-search", has_page);
   gtk_widget_action_set_enabled (GTK_WIDGET (self), "page.zoom-in", has_page);
   gtk_widget_action_set_enabled (GTK_WIDGET (self), "page.zoom-out", has_page);
-  gtk_widget_action_set_enabled (GTK_WIDGET (self), "page.zoom-one", has_page);
   gtk_widget_action_set_enabled (GTK_WIDGET (self), "win.focus-neighbor", has_page);
+
+  /* HACK: Work around GNOME/gtk#7296 by forcing the action on then possibly
+   *       immediately back off.
+   */
+  gtk_widget_action_set_enabled (GTK_WIDGET (self), "page.zoom-one", TRUE);
+  gtk_widget_action_set_enabled (GTK_WIDGET (self), "page.zoom-one", has_page && font_scale != 0);
 }
