@@ -353,6 +353,30 @@ editor_window_close_request (GtkWindow *window)
   return GTK_WINDOW_CLASS (editor_window_parent_class)->close_request (window);
 }
 
+static void
+notify_decoration_layout_cb (EditorWindow *self,
+                             GParamSpec   *pspec,
+                             GtkSettings  *gtk_settings)
+{
+  g_autofree char *layout = NULL;
+  gboolean inverted = FALSE;
+  const char *colon;
+  const char *close_;
+
+  g_assert (EDITOR_IS_WINDOW (self));
+  g_assert (GTK_IS_SETTINGS (gtk_settings));
+
+  g_object_get (gtk_settings,
+                "gtk-decoration-layout", &layout,
+                NULL);
+
+  if ((colon = strchr (layout, ':')) && (close_ = strstr (layout, "close")))
+    inverted = close_ < colon;
+
+  if (self->tab_bar)
+    adw_tab_bar_set_inverted (self->tab_bar, inverted);
+}
+
 static void
 editor_window_constructed (GObject *object)
 {
@@ -362,6 +386,7 @@ editor_window_constructed (GObject *object)
   GtkWidget *zoom_box;
   GtkWidget *zoom_out;
   GtkWidget *zoom_in;
+  GtkSettings *gtk_settings;
 
   g_assert (EDITOR_IS_WINDOW (self));
 
@@ -425,6 +450,14 @@ editor_window_constructed (GObject *object)
   gtk_box_append (GTK_BOX (zoom_box), self->zoom_label);
   gtk_box_append (GTK_BOX (zoom_box), zoom_in);
   gtk_popover_menu_add_child (GTK_POPOVER_MENU (popover), zoom_box, "zoom");
+
+  gtk_settings = gtk_settings_get_default ();
+  g_signal_connect_object (gtk_settings,
+                           "notify::gtk-decoration-layout",
+                           G_CALLBACK (notify_decoration_layout_cb),
+                           self,
+                           G_CONNECT_SWAPPED);
+  notify_decoration_layout_cb (self, NULL, gtk_settings);
 }
 
 static gboolean
@@ -660,29 +693,6 @@ transform_window_title (GBinding     *binding,
   return TRUE;
 }
 
-static void
-on_show_help_overlay_cb (GtkWidget  *widget,
-                         const char *action_name,
-                         GVariant   *param)
-{
-  g_autoptr(GtkBuilder) builder = NULL;
-  GObject *help_overlay;
-
-  g_assert (EDITOR_IS_WINDOW (widget));
-
-  builder = gtk_builder_new_from_resource ("/org/gnome/TextEditor/ui/help-overlay.ui");
-  help_overlay = gtk_builder_get_object (builder, "help_overlay");
-
-  if (GTK_IS_SHORTCUTS_WINDOW (help_overlay))
-    {
-#if DEVELOPMENT_BUILD
-      gtk_widget_add_css_class (GTK_WIDGET (help_overlay), "devel");
-#endif
-      gtk_window_set_transient_for (GTK_WINDOW (help_overlay), GTK_WINDOW (widget));
-      gtk_window_present (GTK_WINDOW (help_overlay));
-    }
-}
-
 static gboolean
 title_query_tooltip_cb (EditorWindow *self,
                         int           x,
@@ -912,7 +922,6 @@ editor_window_class_init (EditorWindowClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, on_tab_view_create_window_cb);
   gtk_widget_class_bind_template_callback (widget_class, title_query_tooltip_cb);
 
-  gtk_widget_class_install_action (widget_class, "win.alternate-help-overlay", NULL, on_show_help_overlay_cb);
   gtk_widget_class_install_action (widget_class, "win.undo-close-page", NULL, on_undo_close_page_cb);
   gtk_widget_class_install_action (widget_class, "win.fullscreen", NULL, editor_window_fullscreen_action);
   gtk_widget_class_install_action (widget_class, "win.unfullscreen", NULL, editor_window_unfullscreen_action);
